//! Domain of dialogs – conversations between two participants.
//!
//! Dialogs are different from rooms in that they are always between two participants.
//! There are no

use std::collections::HashMap;
use std::sync::Arc;

use tokio::sync::RwLock as AsyncRwLock;

use crate::player::PlayerId;
use crate::repo::Storage;

/// Id of a conversation between two players.
///
/// Dialogs are identified by the pair of participants' ids. The order of ids does not matter.
#[derive(Debug, Clone, PartialEq, Eq, Hash)]
pub struct DialogId(PlayerId, PlayerId);
impl DialogId {
    pub fn new(a: PlayerId, b: PlayerId) -> DialogId {
        if a.as_inner() < b.as_inner() {
            DialogId(a, b)
        } else {
            DialogId(b, a)
        }
    }

    pub fn as_inner(&self) -> (&PlayerId, &PlayerId) {
        (&self.0, &self.1)
    }

    pub fn into_inner(self) -> (PlayerId, PlayerId) {
        (self.0, self.1)
    }
}

struct Dialog {
    storage_id: u32,
    id: DialogId,
    message_count: u32,
}

struct DialogRegistryInner {
    dialogs: HashMap<DialogId, AsyncRwLock<Dialog>>,
    storage: Storage,
}

#[derive(Clone)]
pub struct DialogRegistry(Arc<AsyncRwLock<DialogRegistryInner>>);

impl DialogRegistry {
    pub fn new(storage: Storage) -> DialogRegistry {
        DialogRegistry(Arc::new(AsyncRwLock::new(DialogRegistryInner {
            dialogs: HashMap::new(),
            storage,
        })))
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_dialog_id_new() {
        let a = PlayerId::from("a").unwrap();
        let b = PlayerId::from("b").unwrap();
        let id1 = DialogId::new(a.clone(), b.clone());
        let id2 = DialogId::new(a.clone(), b.clone());
        // Dialog ids are invariant with respect to the order of participants
        assert_eq!(id1, id2);
        assert_eq!(id1.as_inner(), (&a, &b));
        assert_eq!(id2.as_inner(), (&a, &b));
    }
}
